#include "common.h"
#include "id3genres.h"

void echo_tag(tag_t *id3tag)
{
	printf("%s:\n"
	       "\tSong:   %s\n"
	       "\tArtist: %s\n", id3tag->filename, id3tag->songname, 
		id3tag->artist); 

	if (id3tag->tracknum) 
		printf("\tAlbum:  %s (track %02u)\n", id3tag->album, 
					id3tag->tracknum);
	else printf("\tAlbum:  %s \n", id3tag->album);	
	
	printf ("\tYear:   %s\n"
		"\tGenre:  0x%02X (%s)\n"
		"\tComm.:  %s\n",
		id3tag->year, id3tag->genre,
		genre_str_by_id(id3tag->genre), id3tag->comment);
}

void list_genres()
{
	int i;

	for (i=0; i<=MAX_GENRE; i++) printf("%3d: %s\n", i, genres[i]);
}

char *genre_str_by_id (unsigned char genre)
{
	return (genre <= MAX_GENRE) ? genres[genre] : strdup("Unknown");
}

unsigned char genre_id_by_str (char *str)
{
	unsigned char i;
	
	for (i=0; i<=MAX_GENRE; i++) 
	{
		if (strcasecmp(genres[i],str) == 0)
		{
			return i;
		}
	}	

	fprintf(stderr,"Warning: `%s': genre not found\n",str);
	return 12; /* genre 12 is other */
}	

tag_t *read_id3(FILE *mp3file, char *filename)
{
	tag_t *id3tag;
	int i;

	id3tag = malloc(sizeof(tag_t));
	memset(id3tag, 0, sizeof(tag_t));

	if (count_id3(mp3file) <= 0) return NULL;	
	if ((i = count_id3(mp3file)) > 1) {
			fprintf(stderr, "%s: WARNING: %d tags\n", filename, i);
	}
	
	if (fseek(mp3file, -125, SEEK_END) != 0) 
	{
		if (id3tag->filename[0])
			fprintf(stderr, "%s: ", id3tag->filename);
		perror("fseek");
		free(id3tag);
		return READ_SEEK_FAILED;
	}	
	
	fread(id3tag->songname, 30, 1, mp3file);	
	fread(id3tag->artist, 30, 1, mp3file);	
	fread(id3tag->album, 30, 1, mp3file);	
	fread(id3tag->year, 4, 1, mp3file);	
	
	fread(id3tag->comment, 30, 1, mp3file);	
	if (id3tag->comment[28] == 0 && id3tag->comment[29] != 0) 
		id3tag->tracknum = id3tag->comment[29];
		/* the line above handles ID3v1.1 track numbers */
	
	fread(&(id3tag->genre), 1, 1, mp3file);	
	
	return id3tag;		
}

void write_id3(FILE *mp3file, tag_t *id3tag)
{
	char buf[3];
	
	if (id3tag->write_method == TAG_APPEND)
	{
		if (verbose) 
			printf("\n\tWriting tag in APPEND (create) mode...\n");
		if (0 != fseek(mp3file, 0, SEEK_END))
		{
			if (id3tag->filename[0])
				fprintf(stderr, "%s: ", id3tag->filename);
			perror("fseek");
			return;
		}	
	}	
	else if (id3tag->write_method == TAG_REPLACE)
	{
		if (verbose) 
			printf("\n\tWriting tag in REPLACE (modify) mode...\n");
		if (0 != fseek(mp3file, -128, SEEK_END))
		{
			if (id3tag->filename[0])
				fprintf(stderr, "%s: ", id3tag->filename);
			perror("fseek");
			return;
		}	
	}
	else 
	{
		fprintf(stderr,"wtf?!! id3tag->write_method has weird value\n");
		fclose(mp3file);
		abort();
	}
	
	strncpy(buf, "TAG", 3);
	fwrite(buf, 3, 1, mp3file);
	
	fwrite(id3tag->songname, 30, 1, mp3file);	
	fwrite(id3tag->artist, 30, 1, mp3file);	
	fwrite(id3tag->album, 30, 1, mp3file);	
	fwrite(id3tag->year, 4, 1, mp3file);	
	
	
	if (id3tag->tracknum == 0)  
		fwrite(id3tag->comment, 30, 1, mp3file);
	else {
		fwrite(id3tag->comment, 28, 1, mp3file);
		buf[0] = 0;
		fwrite(buf, 1, 1, mp3file);
		fwrite(&(id3tag->tracknum), 1, 1, mp3file);
	}	
		/* that piece of c0de handles ID3v1.1 track numbers :) */
	
	fwrite(&(id3tag->genre), 1, 1, mp3file);	
}

int count_id3(FILE *mp3file)
{
	char buf[3];
	int i = 0;
	
	fseek(mp3file, 0, SEEK_END);

	while (ftell(mp3file) >= 128)
	{
		fseek(mp3file, -128, SEEK_CUR);
		fread(buf, 3, 1, mp3file);
		fseek(mp3file, -3, SEEK_CUR);
	        if (strncmp(buf, "TAG", 3) == 0) i++;
		else return i;
	}	
	return (int) READ_SEEK_FAILED;
}
