#include <assert.h>
#include "arg_parser.h"
#include "common.h"

int verbose;

void print_usage(char *program_name)
{
	printf("popol %s, a bug-free ID3v1.1 tagger by XSFX@linuxfreak.com\n"
		"compiled %s %s\n"
		"usage:\t%s [options] mp3_files\n\n",
		POPOL_VERSION, __DATE__, __TIME__, program_name);
	gaa_help();	
	exit(1);	
}	

int main (int argc, char **argv)
{
	FILE *mp3file;
	gaainfo gaaval;
	tag_t *id3tag;
	int i;

	if (argc == 1) print_usage(argv[0]);	
	if (gaa(argc, argv, &gaaval) != -1) exit(1); 
		/* gaa() returns -1 on success */

	verbose = gaaval.verbose;		

/* are we only reading tags  ?? */

	if (gaaval.write_mode == 0)  
	for (i=0; i < gaaval.files_num; i++)
	{
		mp3file = openvalidmp3(gaaval.files[i],"r");
		if (mp3file)
		{
			id3tag = read_id3(mp3file, gaaval.files[i]);

			if (!id3tag || id3tag == READ_SEEK_FAILED) 
			   fprintf(stderr,"%s: no ID3v1 tag found\n\n",
					gaaval.files[i]);
			else {
				strncpy(id3tag->filename, gaaval.files[i],
					sizeof(id3tag->filename) - 1);
				echo_tag(id3tag);
				putchar('\n'); 
				free(id3tag);
			}
			
			fclose(mp3file);
		}
	} 

/* ok then we must be tagging ! */

	else for (i=0; i < gaaval.files_num; i++)
	{ 		      
		mp3file = openvalidmp3(gaaval.files[i],"r+");
		if (mp3file)
		{
			id3tag = read_id3(mp3file, gaaval.files[i]);
			if (id3tag == NULL) 
			{ 
				id3tag = malloc(sizeof(tag_t));
				memset(id3tag, 0, sizeof(tag_t)); 
				id3tag->write_method = TAG_APPEND;
				id3tag->genre = 255;
			}						
			else id3tag->write_method = TAG_REPLACE;
			
			if (gaaval.songname) 
				strncpy(id3tag->songname, gaaval.songname,30);
			if (gaaval.artist)
				strncpy(id3tag->artist, gaaval.artist,30);
			if (gaaval.album)
				strncpy(id3tag->album, gaaval.album,30);
			if (gaaval.comment)
				strncpy(id3tag->comment, gaaval.comment,30);
			if (gaaval.year)
				strncpy(id3tag->year, gaaval.year,4);
			if (gaaval.want_track)
				id3tag->tracknum = gaaval.tracknum;
			
			if (gaaval.want_genre) 
				id3tag->genre = gaaval.genre;
			else if (gaaval.strgenre)
			   id3tag->genre = genre_id_by_str(gaaval.strgenre);
				
			strncpy(id3tag->filename, gaaval.files[i],
				sizeof(id3tag->filename) - 1);

			if (verbose) echo_tag (id3tag);
			write_id3(mp3file, id3tag);
			if (verbose) putchar('\n');
			
			free(id3tag);
			fclose(mp3file);
		}	
		
	}
	
	exit(0);
}
